-- -*- Mode: Lua; lua-indent-level: 4 -*-

local L = CatsEye_Localization

local barTypes = {
    berserk = {
        spell = 50334,
        alternates = {},
        combos = 0,
        aura = true,
        fromAny = false,
    },
    incarnation = {
        spell = 102543,
        alternates = {},
        combos = 0,
        aura = true,
        fromAny = false,
    },
    savageRoar = {
        spell = 52610,
        alternates = {},
        combos = 1,
        aura = true,
        fromAny = false,
    },
    rip = {
        spell = 1079,
        alternates = {},
        combos = 1,
        aura = true,
        fromAny = false,
    },
    mangleSpam = {
        spell = 33876,
        alternates = {},
        combos = 0,
        aura = false,
        fromAny = false,
    },
    rake = {
        spell = 1822,
        alternates = {},
        combos = 0,
        aura = true,
        fromAny = false,
    },
    aounce = {
        spell = 9005,
        alternates = {},
        combos = 0,
        aura = true,
        fromAny = false,
    },
    ferociousBite = {
        spell = 22568,
        alternates = {},
        combos = 1,
        aura = false,
        fromAny = false,
    },
    tigersFury = {
        spell = 5217,
        alternates = {},
        combos = 0,
        aura = true,
        fromAny = false,
    },
    shred = {
        spell = 5221,
        alternates = {},
        combos = 0,
        aura = false,
        fromAny = false,
    },
    faerieFire = {
        spell = 770,
        alternates = {102355},
        combos = 0,
        aura = true,
        fromAny = true,
    },
    weakenedArmor = {
        spell = 113746,
        alternates = {},
        combos = 0,
        aura = true,
        fromAny = true,
    },
    swipe = {
        spell = 62078,
        alternates = {},
        combos = 0,
        aura = false,
        fromAny = false,
    },
    maim = {
        spell = 22570,
        alternates = {},
        combos = 1,
        aura = true,
        fromAny = false,
    },
    ravage = {
        spell = 6785,
        alternates = {},
        combos = 0,
        aura = false,
        fromAny = false,
    },
    pounce = {
        spell = 9005,
        alternates = {9007},
        combos = 0,
        aura = true,
        fromAny = false,
    },
    skullBash = {
        spell = 80965,
        alternates = { 29703, 2139, 1766, 6552, 16979, 47528, 57994, 80964 },
        combos = 0,
        aura = true,
        fromAny = true,
    },
    infectedWounds = {
        spell = 48484,
        alternates = { 55095, 53695, 8042, 6343 },
        combos = 0,
        aura = true,
        fromAny = true,
    },
}

local frame = CreateFrame( "Frame", nil, UIParent )
local power = frame:CreateFontString( nil, "OVERLAY", "GameFontHighlightSmall" )
local combos = frame:CreateTexture( nil, "OVERLAY" )
local omen = frame:CreateTexture( nil, "OVERLAY" )
local playerHealthBG = frame:CreateTexture( nil, "ARTWORK" )
local targetHealthBG = frame:CreateTexture( nil, "ARTWORK" )
local playerHealth = frame:CreateTexture( nil, "OVERLAY" )
local targetHealth = frame:CreateTexture( nil, "OVERLAY" )
local times = {}
local bars = {}
local textureToBars = {}
local berserkUp = false
local inCombat
local normalRipDuration
local barWidth
local autoHide
local showGCD
local unlocked
local maxRemaining
local minDuration
local activeColor
local rechargingColor
local readyColor
local numberOfBars
local healthHeight

local function message( message, ... )
    local line = string.format( message, ... )
    DEFAULT_CHAT_FRAME:AddMessage( line, 1.0, 0.8, 0.4 )
end

local function clamp( numerator, denominator )
    if numerator >= denominator then
        return 1.0
    elseif numerator < 0.0 then
        return 0.0
    end
    return numerator / denominator
end

local function defaults()
    local settings = {
        maxRemaining = 15.0,
        minDuration = 6.0,
        barWidth = 80,
        barHeight = 10,
        barStyle = "roll",
        padding = 3,
        textSize = 8,
        alpha = 0.9,
        autoHide = true,
        showGCD = false,
        unlocked = false,
        activeColor = { 0, 1, 0 },
        rechargingColor = { 1, 0, 0 },
        readyColor = { 0, 0.4, 1 },
        healthColor = { 0, 1, 0 },
        point = "CENTER",
        relativePoint = "CENTER",
        xOffset = 0,
        yOffset = 0,
        bars = {
            "savageRoar",
            "rip",
            "rake",
            "tigersFury",
            "weakenedArmor",
            "incarnation",
            "berserk",
            "-----",
            "faerieFire",
            "pounce",
            "mangleSpam",
            "ferociousBite",
            "shred",
            "ravage",
            "swipe",
            "maim",
            "skullBash",
            "infectedWounds",
        }
    }
    CatsEye_Config = CatsEye_Config or {}
    for key, value in pairs( settings ) do
        if CatsEye_Config[ key ] == nil then
            CatsEye_Config[ key ] = value
        end
    end
    local present = {}
    for _, action in pairs( CatsEye_Config[ "bars" ] ) do
        present[ action ] = true
    end
    for _, action in pairs( settings[ "bars" ] ) do
        if present[ action ] == nil then
            table.insert( CatsEye_Config[ "bars" ], action )
        end
    end
end

local function precomputeTimes()
    for index = 0, 3000 do
        local minutes = math.floor( index / 600 )
        local seconds = ( index - minutes * 600 ) / 10
        if minutes >= 1 then
            times[ index ] = string.format( L.longTime, minutes, seconds )
        elseif seconds >= 10 then
            times[ index ] = string.format( L.mediumTime, seconds )
        else
            times[ index ] = string.format( L.shortTime, seconds )
        end
    end
end

local function startDrag()
    frame:StartMoving()
end

local function stopDrag()
    frame:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint( 1 )
    CatsEye_Config.point = point
    CatsEye_Config.relativePoint = relativePoint
    CatsEye_Config.xOffset = xOffset
    CatsEye_Config.yOffset = yOffset
end

local function update()
    local time = GetTime()
    local energy = UnitPower( "player", SPELL_POWER_ENERGY ) * 0.1
    local combos = GetComboPoints( "player" )
    for index = 1, numberOfBars do
        local bar = bars[ index ]
        if bar.state == 1 then
            local remaining = bar.expiration - time
            local width = clamp( remaining, bar.duration )
            bar.foreground:SetWidth( barWidth * width + 0.001 )
            bar.foreground:SetTexCoord( 0, width, 0, 1 )
            if remaining <= maxRemaining and bar.duration >= minDuration then
                bar.clock:SetText( times[ math.floor( remaining * 10 ) ] or "" )
            end
            if width == 1.0 then
                local start, duration = GetSpellCooldown( bar.name )
                start = start or 0.0
                duration = duration or 0.0
                if duration <= 1.5 and not showGCD then
                    duration = 0.0
                end
                if start > 0.0 and duration > 0.0 or energy < bar.cost or combos < bar.combos then
                    local width = clamp( time - start, duration ) * clamp( energy, bar.cost )
                    bar.state = 2
                    bar.start = start
                    bar.duration = duration
                    bar.background:SetVertexColor( unpack( rechargingColor ) )
                    bar.foreground:SetVertexColor( unpack( rechargingColor ) )
                    bar.foreground:SetWidth( barWidth * width + 0.001 )
                    bar.foreground:SetTexCoord( 0, width, 0, 1 )
                    bar.icon:SetTexture( bar.texture )
                    bar.label:SetText( bar.name )
                    local cooldown = duration - time + start
                    local regenerate = bar.cost - energy
                    local remaining = regenerate > cooldown and regenerate or cooldown
                    duration = bar.cost > duration and bar.cost or duration
                    if remaining <= maxRemaining and duration >= minDuration then
                        bar.clock:SetText( times[ math.floor( remaining * 10 ) ] or "" )
                    else
                        bar.clock:SetText( "" )
                    end
                else
                    bar.state = 0
                    bar.background:SetVertexColor( unpack( readyColor ) )
                    bar.foreground:SetVertexColor( unpack( readyColor ) )
                    bar.icon:SetTexture( bar.texture )
                    bar.label:SetText( bar.name )
                    bar.clock:SetText( "" )
                end
            end
        elseif bar.state == 2 then
            local width = clamp( time - bar.start, bar.duration ) * clamp( energy, bar.cost )
            bar.foreground:SetWidth( barWidth * width + 0.001 )
            bar.foreground:SetTexCoord( 0, width, 0, 1 )
            local cooldown = bar.duration - time + bar.start
            local regenerate = bar.cost - energy
            local remaining = regenerate > cooldown and regenerate or cooldown
            local duration = bar.cost > bar.duration and bar.cost or bar.duration
            if remaining <= maxRemaining and duration >= minDuration then
                bar.clock:SetText( times[ math.floor( remaining * 10 ) ] or "" )
            end
            if width == 1.0 and combos >= bar.combos then
                bar.state = 0
                bar.background:SetVertexColor( unpack( readyColor ) )
                bar.foreground:SetVertexColor( unpack( readyColor ) )
                bar.icon:SetTexture( bar.texture )
                bar.label:SetText( bar.name )
                bar.clock:SetText( "" )
            end
        end
    end
end

local function showOrHide()
    local _, _, inCat = GetShapeshiftFormInfo( 3 )
    if inCombat and inCat or unlocked or not autoHide then
        frame:RegisterEvent( "UNIT_HEALTH" )
        frame:RegisterEvent( "UNIT_MAXHEALTH" )
        frame:RegisterEvent( "UNIT_AURA" )
        frame:RegisterEvent( "UNIT_AURASTATE" )
        frame:RegisterEvent( "SPELL_UPDATE_COOLDOWN" )
        frame:RegisterEvent( "UNIT_COMBO_POINTS" )
        frame:RegisterEvent( "UNIT_ATTACK_POWER" )
        frame:RegisterEvent( "PLAYER_TARGET_CHANGED" )
        frame:SetScript( "OnUpdate", update )
        frame:Show()
    else
        frame:UnregisterEvent( "UNIT_HEALTH" )
        frame:UnregisterEvent( "UNIT_MAXHEALTH" )
        frame:UnregisterEvent( "UNIT_AURA" )
        frame:UnregisterEvent( "UNIT_AURASTATE" )
        frame:UnregisterEvent( "SPELL_UPDATE_COOLDOWN" )
        frame:UnregisterEvent( "UNIT_COMBO_POINTS" )
        frame:UnregisterEvent( "UNIT_ATTACK_POWER" )
        frame:UnregisterEvent( "PLAYER_TARGET_CHANGED" )
        frame:SetScript( "OnUpdate", nil )
        frame:Hide()
    end
end

local function updateCosts()
    for index = 1, numberOfBars do
        local type = CatsEye_Config.bars[ index ]
        local _, _, _, cost = GetSpellInfo( barTypes[ type ].spell )
        bars[ index ] = bars[ index ] or {}
        bars[ index ].cost = cost * 0.1 or 0
    end
end

local function setup()
    local padding = CatsEye_Config.padding
    local barHeight = CatsEye_Config.barHeight
    local barStyle = CatsEye_Config.barStyle
    local textSize = CatsEye_Config.textSize
    local alpha = CatsEye_Config.alpha
    local point = CatsEye_Config.point
    local relativePoint = CatsEye_Config.relativePoint
    local xOffset = CatsEye_Config.xOffset
    local yOffset = CatsEye_Config.yOffset
    local healthColor = CatsEye_Config.healthColor
    barWidth = CatsEye_Config.barWidth
    autoHide = CatsEye_Config.autoHide
    showGCD = CatsEye_Config.showGCD
    unlocked = CatsEye_Config.unlocked
    maxRemaining = CatsEye_Config.maxRemaining
    minDuration = CatsEye_Config.minDuration
    activeColor = CatsEye_Config.activeColor
    rechargingColor = CatsEye_Config.rechargingColor
    readyColor = CatsEye_Config.readyColor
    numberOfBars = 0
    for _, type in ipairs( CatsEye_Config.bars ) do
        if type == "-----" then
            break
        end
        numberOfBars = numberOfBars + 1
    end
    healthHeight = ( barHeight + padding ) * numberOfBars - padding
    frame:SetWidth( barWidth + barHeight + padding * 7 )
    frame:SetHeight( ( barHeight + padding ) * ( numberOfBars + 1 ) + padding )
    frame:ClearAllPoints()
    frame:SetPoint( point, UIParent, relativePoint, xOffset, yOffset )
    frame:SetBackdrop( { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
                         edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
                         tile = true,
                         tileSize = 8,
                         edgeSize = 8,
                         insets = { left = 2, right = 2, top = 2, bottom = 2 } } )
    frame:SetBackdropColor( 1, 1, 1, 1 )
    frame:SetBackdropBorderColor( 1, 1, 1, 1 )
    frame:SetFrameStrata( "MEDIUM" )
    frame:SetAlpha( alpha )
    frame:RegisterForDrag( unlocked and "LeftButton" or nil )
    frame:SetMovable( unlocked )
    frame:EnableMouse( unlocked )
    frame:SetScript( "OnDragStart", unlocked and startDrag or nil )
    frame:SetScript( "OnDragStop", unlocked and stopDrag or nil )
    showOrHide()
    power:SetFont( "Fonts\\FRIZQT__.TTF", textSize )
    power:SetPoint( "TOPLEFT", frame, "TOPLEFT", padding, -padding )
    power:SetText( "0" )
    combos:SetWidth( barHeight * 106 / 18 )
    combos:SetHeight( barHeight )
    combos:SetTexture( "Interface\\AddOns\\CatsEye\\Textures\\Combos" )
    combos:SetTexCoord( 0, 106 / 128, 0, 18 / 128 )
    combos:SetPoint( "TOPRIGHT", frame, "TOPRIGHT", -padding, -padding )
    omen:SetWidth( barHeight )
    omen:SetHeight( barHeight )
    omen:SetTexture( "Interface\\AddOns\\CatsEye\\Textures\\Omen" )
    omen:SetPoint( "RIGHT", combos, "LEFT", -padding, 0 )
    omen:Hide()
    playerHealthBG:SetWidth( padding )
    playerHealthBG:SetHeight( healthHeight )
    playerHealthBG:SetTexture( unpack( healthColor ) )
    playerHealthBG:SetGradientAlpha( "VERTICAL", 0.25, 0.25, 0.25, 0.05, 1, 1, 1, 0.25 )
    playerHealthBG:SetPoint( "BOTTOMLEFT", frame, "BOTTOMLEFT", padding, padding )
    targetHealthBG:SetWidth( padding )
    targetHealthBG:SetHeight( healthHeight )
    targetHealthBG:SetTexture( unpack( healthColor ) )
    targetHealthBG:SetGradientAlpha( "VERTICAL", 0.25, 0.25, 0.25, 0.05, 1, 1, 1, 0.25 )
    targetHealthBG:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", -padding, padding )
    playerHealth:SetWidth( padding )
    playerHealth:SetHeight( healthHeight )
    playerHealth:SetTexture( unpack( healthColor ) )
    playerHealth:SetGradientAlpha( "VERTICAL", 0.25, 0.25, 0.25, 0.25, 1, 1, 1, 1 )
    playerHealth:SetPoint( "BOTTOMLEFT", frame, "BOTTOMLEFT", padding, padding )
    targetHealth:SetWidth( padding )
    targetHealth:SetHeight( healthHeight )
    targetHealth:SetTexture( unpack( healthColor ) )
    targetHealth:SetGradientAlpha( "VERTICAL", 0.25, 0.25, 0.25, 0.25, 1, 1, 1, 1 )
    targetHealth:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", -padding, padding )
    for index, bar in ipairs( bars ) do
        if bar.background then
            bar.background:Hide()
            bar.foreground:Hide()
            bar.icon:Hide()
            bar.label:Hide()
        end
    end
    textureToBars = {}
    
    for index, type in ipairs( CatsEye_Config.bars ) do
        if type == "-----" then
            break
        end
        bars[ index ] = bars[ index ] or {}
        local bar = bars[ index ]
        local name, _, texture, cost = GetSpellInfo( barTypes[ type ].spell )
        bar.name = name
        bar.texture = texture
        bar.cost = cost * 0.1 or 0
        bar.combos = barTypes[ type ].combos
        bar.fromAny = barTypes[ type ].fromAny
        bar.state = 0
        bar.background = bar.background or frame:CreateTexture( nil, "ARTWORK" )
        bar.background:SetWidth( barWidth )
        bar.background:SetHeight( barHeight )
        bar.background:SetTexture( "Interface\\AddOns\\CatsEye\\Textures\\BarBG-" .. string.gsub( barStyle, "^%l", string.upper ) )
        bar.background:SetVertexColor( unpack( readyColor ) )
        bar.background:SetTexCoord( 0, 1, 0, 1 )
        if index == 1 then
            bar.background:SetPoint( "TOPRIGHT", targetHealthBG, "TOPLEFT", -padding, 0 )
        else
            bar.background:SetPoint( "TOP", bars[ index - 1 ].background, "BOTTOM", 0, -padding )
        end
        bar.background:Show()
        bar.foreground = bar.foreground or frame:CreateTexture( nil, "OVERLAY" )
        bar.foreground:SetWidth( barWidth )
        bar.foreground:SetHeight( barHeight )
        bar.foreground:SetTexture( "Interface\\AddOns\\CatsEye\\Textures\\BarFG-" .. string.gsub( barStyle, "^%l", string.upper ) )
        bar.foreground:SetVertexColor( unpack( readyColor ) )
        bar.foreground:SetTexCoord( 0, 1, 0, 1 )
        bar.foreground:SetPoint( "TOPLEFT", bar.background, "TOPLEFT" )
        bar.foreground:Show()
        bar.icon = bar.icon or frame:CreateTexture( nil, "OVERLAY" )
        bar.icon:SetWidth( barHeight )
        bar.icon:SetHeight( barHeight )
        bar.icon:SetTexture( texture )
        bar.icon:SetPoint( "RIGHT", bar.background, "LEFT", -padding, 0 )
        bar.icon:Show()
        bar.clock = bar.clock or frame:CreateFontString( nil, "OVERLAY", "GameFontHighlightSmall" )
        bar.clock:SetFont( "Fonts\\FRIZQT__.TTF", textSize )
        bar.clock:SetPoint( "RIGHT", bar.background, "RIGHT", -padding, 0 )
        bar.clock:SetText( "" )
        bar.clock:Show()
        bar.label = bar.label or frame:CreateFontString( nil, "OVERLAY", "GameFontHighlightSmall" )
        bar.label:SetFont( "Fonts\\FRIZQT__.TTF", textSize )
        bar.label:SetPoint( "LEFT", bar.background, "LEFT", padding, 0 )
        bar.label:SetPoint( "RIGHT", bar.clock, "LEFT", -padding, 0 )
        bar.label:SetJustifyH( "LEFT" )
        bar.label:SetText( name )
        bar.label:Show()
        if barTypes[ type ].aura then
            textureToBars[ texture ] = textureToBars[ texture ] or {}
            table.insert( textureToBars[ texture ], bar )
            for _, alternate in ipairs( barTypes[ type ].alternates ) do
                local _, _, texture = GetSpellInfo( alternate )
                if texture then
                    textureToBars[ texture ] = textureToBars[ texture ] or {}
                    table.insert( textureToBars[ texture ], bar )
                end
            end
        end
    end
end

local function updateHealth( bar, unit )
    if UnitIsDead( unit ) then
        bar:SetHeight( 0.001 )
    else
        local health = UnitHealth( unit )
        local maximum = UnitHealthMax( unit )
        bar:SetHeight( healthHeight * health / maximum )
    end
end

local function scan()
    local time = GetTime()
    local energy = UnitPower( "player", SPELL_POWER_ENERGY ) * 0.1
    local combo = GetComboPoints( "player" )
    local offset = combo * 22 / 128
    combos:SetTexCoord( 0, 106 / 128, offset, offset + 18 / 128 )
    local base, buffed, debuffed = UnitAttackPower( "player" )
    power:SetText( base + buffed + debuffed )
    for index = 1, numberOfBars do
        local bar = bars[ index ]
        local start, duration = GetSpellCooldown( bar.name )
        start = start or 0.0
        duration = duration or 0.0
        if duration <= 1.5 and not showGCD then
            duration = 0.0
        end
        if start > 0.0 and duration > 0.0 or energy < bar.cost or combo < bar.combos then
            local width = clamp( time - start, duration ) * clamp( energy, bar.cost )
            bar.state = 2
            bar.start = start
            bar.duration = duration
            bar.background:SetVertexColor( unpack( rechargingColor ) )
            bar.foreground:SetVertexColor( unpack( rechargingColor ) )
            bar.foreground:SetWidth( barWidth * width + 0.001 )
            bar.foreground:SetTexCoord( 0, width, 0, 1 )
            bar.icon:SetTexture( bar.texture )
            bar.label:SetText( bar.name )
            local cooldown = duration - time + start
            local regenerate = bar.cost - energy
            local remaining = regenerate > cooldown and regenerate or cooldown
            duration = bar.cost > duration and bar.cost or duration
            if remaining <= maxRemaining and duration >= minDuration then
                bar.clock:SetText( times[ math.floor( remaining * 10 ) ] or "" )
            else
                bar.clock:SetText( "" )
            end
        else
            bar.state = 0
            bar.background:SetVertexColor( unpack( readyColor ) )
            bar.foreground:SetVertexColor( unpack( readyColor ) )
            bar.foreground:SetWidth( barWidth )
            bar.foreground:SetTexCoord( 0, 1, 0, 1 )
            bar.icon:SetTexture( bar.texture )
            bar.label:SetText( bar.name )
            bar.clock:SetText( "" )
        end
    end
    if not UnitIsDead( "target" ) then
        for index = 1, 40 do
            local name, _, texture, _, _, duration, expiration, from = UnitDebuff( "target", index )
            if not name then
                break
            end
            local mine = from == "player"
            if texture == "Interface\\Icons\\Ability_GhoulFrenzy" and mine then
                if not normalRipDuration then
                    normalRipDuration = duration
                elseif duration < normalRipDuration then
                    duration = normalRipDuration
                end
            end
            local barsToUpdate = textureToBars[ texture ]
            if barsToUpdate then
                for _, bar in ipairs( barsToUpdate ) do
                    if ( bar.state ~= 1 or expiration > bar.expiration ) and ( mine or bar.fromAny ) then
                        local remaining = expiration - time
                        local width = clamp( remaining, duration )
                        bar.state = 1
                        bar.duration = duration
                        bar.expiration = expiration
                        bar.background:SetVertexColor( unpack( activeColor ) )
                        bar.foreground:SetVertexColor( unpack( activeColor ) )
                        bar.foreground:SetWidth( barWidth * width + 0.001 )
                        bar.foreground:SetTexCoord( 0, width, 0, 1 )
                        bar.icon:SetTexture( texture )
                        bar.label:SetText( name )
                        if remaining <= maxRemaining and duration >= minDuration then
                            bar.clock:SetText( times[ math.floor( remaining * 10 ) ] or "" )
                        else
                            bar.clock:SetText( "" )
                        end
                    end
                end
            end
        end
    end
    omen:Hide()
    local sawBerserk = false
    for index = 1, 40 do
        local name, _, texture, _, _, duration, expiration = UnitBuff( "player", index )
        if not name then
            break
        end
        local barsToUpdate = textureToBars[ texture ]
        if barsToUpdate then
            for _, bar in ipairs( barsToUpdate ) do
                if bar.state ~= 1 or expiration > bar.expiration then
                    local remaining = expiration - time
                    local width = clamp( remaining, duration )
                    bar.state = 1
                    bar.duration = duration
                    bar.expiration = expiration
                    bar.background:SetVertexColor( unpack( activeColor ) )
                    bar.foreground:SetVertexColor( unpack( activeColor ) )
                    bar.foreground:SetWidth( barWidth * width + 0.001 )
                    bar.foreground:SetTexCoord( 0, width, 0, 1 )
                    if remaining <= maxRemaining and duration >= minDuration then
                        bar.clock:SetText( times[ math.floor( remaining * 10 ) ] or "" )
                    else
                        bar.clock:SetText( "" )
                    end
                end
            end
        end
        if texture == "Interface\\Icons\\Spell_Shadow_ManaBurn" then
            omen:Show()
        elseif texture == "Interface\\Icons\\Ability_Druid_Berserk" then
            sawBerserk = true
        end
    end
    if sawBerserk ~= berserkUp then
        updateCosts()
        berserkUp = sawBerserk
    end
end

local function process( this, event, arg1 )
    if event == "UNIT_HEALTH" or event == "UNIT_MAXHEALTH" then
        if arg1 == "player" then
            updateHealth( playerHealth, "player" )
        elseif arg1 == "target" then
            updateHealth( targetHealth, "target" )
        end
    elseif event == "UNIT_AURA" or event == "UNIT_AURASTATE" then
        if arg1 == "player" or arg1 == "target" then
            scan()
        end
    elseif event == "SPELL_UPDATE_COOLDOWN" then
        scan()
    elseif event == "UNIT_COMBO_POINTS" then
        scan()
    elseif event == "UNIT_ATTACK_POWER" then
        local base, buffed, debuffed = UnitAttackPower( "player" )
        power:SetText( base + buffed + debuffed )
    elseif event == "PLAYER_TARGET_CHANGED" then
        updateHealth( targetHealth, "target" )
        scan()
    elseif event == "PLAYER_REGEN_DISABLED" then
        normalRipDuration = nil
        inCombat = true
        showOrHide()
    elseif event == "PLAYER_REGEN_ENABLED" then
        inCombat = false
        showOrHide()
    elseif event == "UPDATE_SHAPESHIFT_FORM" then
        updateCosts()
        scan()
        showOrHide()
    end
end

local function initialize( self, event, ... )
    local name = ...;
    if name == "CatsEye" then
        frame:UnregisterEvent( "ADDON_LOADED" )
        local _, englishClass = UnitClass( "player" )
        if englishClass ~= "DRUID" then
            return
        end
        defaults()
        precomputeTimes()
        setup()
        scan()
        frame:SetScript( "OnEvent", process )
        frame:RegisterEvent( "PLAYER_REGEN_DISABLED" )
        frame:RegisterEvent( "PLAYER_REGEN_ENABLED" )
        frame:RegisterEvent( "UPDATE_SHAPESHIFT_FORM" )
        message( L.loaded )
    end
end

frame:RegisterEvent( "ADDON_LOADED" )
frame:SetScript( "OnEvent", initialize )

CatsEye = {
    defaults = defaults,
    setup = setup,
    scan = scan,
}
